require "spec_helper"
require "toni"
require "ostruct"

describe Toni::PermissionFilter do

  let(:expectation_builder) { Toni::Builder::ExpectationBuilder.new }
  let(:matcher) { expectation_builder.eq("test") }
  let(:permission_matcher) { Toni::PermissionMatcher::Base.new(:foo, matcher, true) }

  let(:permission_filter) do
    described_class.new(permission_matcher)
  end

  describe ".permission_filters" do
    it "returns constants of available permission filters" do
      expect(Toni::PermissionFilter.permission_filters).to match_array([
        :Eq, :BeNil, :BeTruthy, :BeFalsey, :BeBetween
      ])
    end
  end

  describe Toni::PermissionFilter::Eq do
    it "creates condition" do
      expect(permission_filter.filter_condition).to eq(["foo = ?", "test"])
    end
    it "creates opposite condition" do
      expect(permission_filter.filter_condition_inverse).to eq(["foo != ?", "test"])
    end
  end

  describe Toni::PermissionFilter::BeNil do
    it "creates condition" do
      expect(permission_filter.filter_condition).to eq(["foo IS NULL"])
    end
    it "creates opposite condition" do
      expect(permission_filter.filter_condition_inverse).to eq(["foo IS NOT NULL"])
    end
  end

  describe Toni::PermissionFilter::BeTruthy do
    it "creates condition" do
      expect(permission_filter.filter_condition).to eq(["foo IS NOT NULL AND foo != ''"])
    end
    it "creates opposite condition" do
      expect(permission_filter.filter_condition_inverse).to eq(["foo = '' OR foo IS NULL"])
    end
  end

  describe Toni::PermissionFilter::BeFalsey do
    it "creates condition" do
      expect(permission_filter.filter_condition).to eq(["foo = '' OR foo IS NULL"])
    end
    it "creates opposite condition" do
      expect(permission_filter.filter_condition_inverse).to eq(["foo IS NOT NULL AND foo != ''"])
    end
  end

  describe Toni::PermissionFilter::BeBetween do
    let(:matcher) { expectation_builder.be_between(5, 10) }
    it "creates condition" do
      expect(permission_filter.filter_condition).to eq(["foo >= ? AND foo <= ?", 5, 10])
    end
    it "creates opposite condition" do
      expect(permission_filter.filter_condition_inverse).to eq(["foo < ? OR foo > ?", 5, 10])
    end
  end

end