require "spec_helper"
require "toni/rspec_helper"

describe Toni::RSpecHelper do
  include Toni::RSpecHelper

  describe :without_authorization do
    it "calls Toni without_authorization" do
      expect(Toni).to receive(:without_authorization)
      without_authorization
    end
  end

  describe :with_user do
    it "calls block with given user and return the result of the block" do
      user = double("user")
      expect(Toni).to receive(:current_user=).with(user)
      expect(Toni).to receive(:current_user=).with(nil)
      expect(with_user(user) { 1 }).to eq(1)
    end
  end

  describe "have_permission_on matcher" do
    it "returns true if all permissions are true for the given activities" do
      user = double("user", role_symbols: [])
      expect(Toni).to receive(:permitted_to?).with(:create, :books) { true }
      expect(Toni).to receive(:permitted_to?).with(:read, :books) { true }
      expect(user).to have_permission_on(:books, to: [:create, :read])
    end
    it "returns false if one permissions is not true for the given activities" do
      user = double("user", role_symbols: [])
      expect(Toni).to receive(:permitted_to?).with(:create, :books) { true }
      expect(Toni).to receive(:permitted_to?).with(:read, :books) { false }
      expect(user).not_to have_permission_on(:books, to: [:create, :read])
    end
  end
end