require "spec_helper"
require "toni/sinatra"
require "rack/test"

describe Toni::Sinatra::AuthorizationHelper do
  let(:helper) {
    Class.new do
      include Toni::Sinatra::AuthorizationHelper
    end.new
  }

  describe :permitted_to do
    it "calls permitted_to? of toni, sets bang to true and calls the given block" do
      double = double("block")
      expect(Toni).to receive(:permitted_to?).with(:read, :books, bang: true) { true }
      expect(double).to receive(:call)
      helper.permitted_to(:read, :books) do
        double.call
      end
    end

    it "does not call the block if it is not permitted to" do
      double = double("block")
      expect(Toni).to receive(:permitted_to?).with(:read, :books, bang: false) { false }
      expect(double).to_not receive(:call)
      helper.permitted_to(:read, :books, bang: false) do
        double.call
      end
    end
  end

  describe :without_authorization do
    it "calls toni without_authorization" do
      expect(Toni).to receive(:without_authorization)
      helper.without_authorization
    end
  end
end

describe Toni::Sinatra::AuthorizationErrorHandler do
  include Rack::Test::Methods

  let(:app) {
    Class.new(Sinatra::Base) do
      register Toni::Sinatra

      set :show_exceptions, false
      set :raise_errors, false

      not_authorized do |e|
        halt 403, e.message
      end

      get '/' do
        permitted_to(:read, :books) do
          "permitted"
        end
      end
    end
  }

  describe :not_authorized do
    it "does not call error handler when permitted to" do
      allow(Toni).to receive(:permitted_to?).with(:read, :books, anything) { true }
      get '/'
      expect(last_response.body).to eq("permitted")
    end

    it "calls error handler when permitted to" do
      allow(Toni).to receive(:permitted_to?).with(:read, :books, anything) do
        raise Toni::NotAuthorizedError.new("not authorized")
      end
      get '/'
      expect(last_response.status).to eq(403)
      expect(last_response.body).to eq("not authorized")
    end
  end
end